#!/usr/bin/perl -w

# Program pingsweep.pl

# (c) Gary C. Kessler, 2013-2015 [gck@garykessler.net]

# Program to ping a specified set of hosts within a Class C address block.

# A few lines of the code are based on multiPing.pl written by thepuregeek

# ********************************************************
# MAIN PROGRAM BLOCK
# ********************************************************

use Net::Ping;

# Initialize global variables

$version = "1.6";
$build_date = "22 December 2015";

# Default values for first and last HOSTID, unless specified in command line.
# Also set the $hit and $miss flags (start both with TRUE)

$first = 1;
$last = 254;
$netid = 0;
$hit = TRUE;
$miss = TRUE;
$num_hits = 0;

#
# Begin program activity!!
#

print "\nPing Sweep v$version - Gary C. Kessler ($build_date)\n\n";

# Parse command for file name and request missing information.

if (parsecommandline ())
  { exit 1; };

if ($netid eq 0)
  {
  print "Missing NETID (-n parameter).\n\n";
  exit 1;
  };

if (validate_addresses ())
  {
  print "Invalid IP address in NETID or first or last HOSTIDs.\n\n";
  valid_address_text();
  exit 1;
  };

if ($hit eq "FALSE" && $miss eq "FALSE")
  {
  print "Can't set both -s and -m flags.\n\n";
  exit 1;
  };

print "Pinging ", $netid, ".", $first, " - ", $netid, ".", $last;
if ($hit eq "FALSE")
  { print " (Only ping misses/failure will be displayed)"; }
if ($miss eq "FALSE")
  { print " (Only ping hits/success will be displayed)"; }
print "\n\n";

$ping_pkt = Net::Ping->new("icmp", 2);

for ($hostid=$first; $hostid<=$last; $hostid++)
  {
  $ipaddr = $netid . "." . $hostid;

  if ($ping_pkt->ping($ipaddr))
      {
      $num_hits++;
      if ($hit eq "TRUE")
        { print "Ping to $ipaddr succeeded on ",scalar(localtime)," \n"; }
      }
    else
      {
      if ($miss eq "TRUE")
        { print "Ping to $ipaddr failed on ",scalar(localtime)," \n"; }
      }
  }

print "\n\n";

$num_hosts = $last - $first + 1;
$hit_rate = $num_hits / $num_hosts * 100;
printf "%d hosts pinged, %d succeeded, %.2f%% success rate.\n\n", $num_hosts, $num_hits, $hit_rate;

exit;

# ********************************************************
# valid_address_text
# ********************************************************

# Display the valid address rules file

sub valid_address_text
{
print<< "EOT";
Valid IP addresses for this program need to conform to the following rules:

1) The NETID (-n parameter) must be three decimal digits separated by a
   dot, in the form 1.2.3
2) The first decimal digit in the NETID must be in the range 1-223; the
   other two digits in the NETID must be in the range 0-254.
3) The first HOSTID to scan (-f parameter) must be in the range 1-254.
4) The last HOSTID to scan (-l parameter) must be in the range first-254.

EOT
return;
}

# ********************************************************
# validate_addresses
# ********************************************************

# Validate $netid, $first, and $last IP addresses

# Return state = 1 to indicate error condition

sub validate_addresses
{
my $i;

# Check $netid. Format must be X.N.N, X=[1,223], N=[0,254], no leading zeroes
# First, split $netid into component pieces. If not three, error out...

my @nets = split /\./, $netid;
if (scalar @nets != 3) { return 1; }

# Now, retrieve each component part; need to examine separately because
# the legal range is different.

for ($i=0; $i<=2; $i++)
  {
  $nets[$i] = abs ($nets[$i] + 0); # This strips out any leading zeros or extraneous characters
  }

# First decimal digit in Class C netid must be 1-223

if ($nets[0] < 1 || $nets[0] > 223) { return 1; }

# Next two decimal digits can be 0-254

for ($i=1; $i<=2; $i++)
  {
  if ($nets[$i] < 0 || $nets[$i] > 254) { return 1; }
  }

$netid = $nets[0] . "." . $nets[1] . "." . $nets[2];

# Check $first. 1<=$first<=254

$first = abs($first + 0);
if ($first < 1 || $first > 254) { return 1; }

# Check $last. $first<=$last<=254

$last = abs($last + 0);
if ($last < $first || $last > 254) { return 1; }

return 0;
}

# ********************************************************
# parsecommandline
# ********************************************************

# Parse command line for parameters, if present. Query
# user for any missing information

# Return $state = 1 to indicate that the program should stop
# immediately (switch -h)

sub parsecommandline
{
my $state = 0;

# Parse command line switches ($ARGV array of length $#ARGV)

if ($#ARGV >= 0)
  { 
  for ($i = 0; $i <= $#ARGV; $i++)
    {
    PARMS:
      {
      $ARGV[$i] eq "-n" && do
         {
         $netid = $ARGV[$i+1];
         $i++;
         last PARMS;
         };
      $ARGV[$i] eq "-f" && do
         {
         $first = $ARGV[$i+1];
         $i++;
         last PARMS;
         };
      $ARGV[$i] eq "-l" && do
         {
         $last = $ARGV[$i+1];
         $i++;
         last PARMS;
         };
      $ARGV[$i] eq "-s" && do
         {
         $miss = FALSE;
         last PARMS;
         };
      $ARGV[$i] eq "-m" && do
         {
         $hit = FALSE;
         last PARMS;
         };
      $ARGV[$i] eq "-h" && do
         {
         help_text();
         $state = 1;
         return $state;
         };

      do
         {
         print "Invalid parameter \"$ARGV[$i]\".\n\n";
         print "Usage: pingsweep.pl -n Class_C_NETID [-f first_HOSTID] [-l last_HOSTID] [-s | -m]\n";
         print "       pingsweep.pl [-h]\n\n";
         $state = 1;
         return $state;
         };
      };
    };
  };

return $state;
}

# ********************************************************
# help_text
# ********************************************************

# Display the help file

sub help_text
{
print<< "EOT";
Program usage: pingsweep.pl -h
               pingsweep.pl -n Class_C_NETID [-f first_HOSTID] [-l last_HOSTID] [-s | -m]

 where: -h prints this help file
        -n is the Class C NETID to sweep, in the form N.N.N
        -f is the first HOSTID in the sweep range; default = 1
        -l is the last HOSTID in the sweep range; default = 254
        -s signals the program to display only ping hits (not misses)
        -m signals the program to display only ping misses (not hits)

NOTE: Be sure to run this program as su or admin.

EOT
return;
}

